@@ -1,5 +1,5 @@
 /* Renesas RL78 specific support for 32-bit ELF.
-   Copyright (C) 2011-2015 Free Software Foundation, Inc.
+   Copyright (C) 2011-2017 Free Software Foundation, Inc.
 
    This file is part of BFD, the Binary File Descriptor library.
 
@@ -15,7 +15,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston,
+   MA 02110-1301, USA.  */
 
 #include "sysdep.h"
 #include "bfd.h"
@@ -45,25 +46,25 @@ static bfd_reloc_status_type rl78_special_reloc (bfd *, arelent *, asymbol *, vo
 
 static reloc_howto_type rl78_elf_howto_table [] =
 {
-  RL78REL (NONE,         3,  0, 0, dont,     FALSE),
-  RL78REL (DIR32,        2, 32, 0, signed,   FALSE),
-  RL78REL (DIR24S,       2, 24, 0, signed,   FALSE),
-  RL78REL (DIR16,        1, 16, 0, dont,     FALSE),
-  RL78REL (DIR16U,       1, 16, 0, unsigned, FALSE),
-  RL78REL (DIR16S,       1, 16, 0, signed,   FALSE),
-  RL78REL (DIR8,         0,  8, 0, dont,     FALSE),
-  RL78REL (DIR8U,        0,  8, 0, unsigned, FALSE),
-  RL78REL (DIR8S,        0,  8, 0, signed,   FALSE),
+  RL78REL (NONE,	 3,  0, 0, dont,     FALSE),
+  RL78REL (DIR32,	 2, 32, 0, signed,   FALSE),
+  RL78REL (DIR24S,	 2, 24, 0, signed,   FALSE),
+  RL78REL (DIR16,	 1, 16, 0, dont,     FALSE),
+  RL78REL (DIR16U,	 1, 16, 0, unsigned, FALSE),
+  RL78REL (DIR16S,	 1, 16, 0, signed,   FALSE),
+  RL78REL (DIR8,	 0,  8, 0, dont,     FALSE),
+  RL78REL (DIR8U,	 0,  8, 0, unsigned, FALSE),
+  RL78REL (DIR8S,	 0,  8, 0, signed,   FALSE),
   RL78REL (DIR24S_PCREL, 2, 24, 0, signed,   TRUE),
   RL78REL (DIR16S_PCREL, 1, 16, 0, signed,   TRUE),
-  RL78REL (DIR8S_PCREL,  0,  8, 0, signed,   TRUE),
-  RL78REL (DIR16UL,      1, 16, 2, unsigned, FALSE),
-  RL78REL (DIR16UW,      1, 16, 1, unsigned, FALSE),
-  RL78REL (DIR8UL,       0,  8, 2, unsigned, FALSE),
-  RL78REL (DIR8UW,       0,  8, 1, unsigned, FALSE),
-  RL78REL (DIR32_REV,    1, 16, 0, dont,     FALSE),
-  RL78REL (DIR16_REV,    1, 16, 0, dont,     FALSE),
-  RL78REL (DIR3U_PCREL,  0,  3, 0, dont,     TRUE),
+  RL78REL (DIR8S_PCREL,	 0,  8, 0, signed,   TRUE),
+  RL78REL (DIR16UL,	 1, 16, 2, unsigned, FALSE),
+  RL78REL (DIR16UW,	 1, 16, 1, unsigned, FALSE),
+  RL78REL (DIR8UL,	 0,  8, 2, unsigned, FALSE),
+  RL78REL (DIR8UW,	 0,  8, 1, unsigned, FALSE),
+  RL78REL (DIR32_REV,	 1, 16, 0, dont,     FALSE),
+  RL78REL (DIR16_REV,	 1, 16, 0, dont,     FALSE),
+  RL78REL (DIR3U_PCREL,	 0,  3, 0, dont,     TRUE),
 
   EMPTY_HOWTO (0x13),
   EMPTY_HOWTO (0x14),
@@ -92,10 +93,10 @@ static reloc_howto_type rl78_elf_howto_table [] =
   EMPTY_HOWTO (0x2a),
   EMPTY_HOWTO (0x2b),
   EMPTY_HOWTO (0x2c),
-  RL78REL (RH_RELAX, 0,  0, 0, dont,     FALSE),
+  RL78REL (RH_RELAX, 0,	 0, 0, dont,	 FALSE),
 
   EMPTY_HOWTO (0x2e),
-  EMPTY_HOWTO (0x2f),
+  RL78REL (RH_SADDR, 0,	 0, 0, dont,	 FALSE),
   EMPTY_HOWTO (0x30),
   EMPTY_HOWTO (0x31),
   EMPTY_HOWTO (0x32),
@@ -114,23 +115,23 @@ static reloc_howto_type rl78_elf_howto_table [] =
   EMPTY_HOWTO (0x3f),
   EMPTY_HOWTO (0x40),
 
-  RL78_OP_REL (ABS32,        2, 32, 0, dont,     FALSE),
-  RL78_OP_REL (ABS24S,       2, 24, 0, signed,   FALSE),
-  RL78_OP_REL (ABS16,        1, 16, 0, dont,     FALSE),
-  RL78_OP_REL (ABS16U,       1, 16, 0, unsigned, FALSE),
-  RL78_OP_REL (ABS16S,       1, 16, 0, signed,   FALSE),
-  RL78_OP_REL (ABS8,         0,  8, 0, dont,     FALSE),
-  RL78_OP_REL (ABS8U,        0,  8, 0, unsigned, FALSE),
-  RL78_OP_REL (ABS8S,        0,  8, 0, signed,   FALSE),
-  RL78_OP_REL (ABS24S_PCREL, 2, 24, 0, signed,   TRUE),
-  RL78_OP_REL (ABS16S_PCREL, 1, 16, 0, signed,   TRUE),
-  RL78_OP_REL (ABS8S_PCREL,  0,  8, 0, signed,   TRUE),
-  RL78_OP_REL (ABS16UL,      1, 16, 0, unsigned, FALSE),
-  RL78_OP_REL (ABS16UW,      1, 16, 0, unsigned, FALSE),
-  RL78_OP_REL (ABS8UL,       0,  8, 0, unsigned, FALSE),
-  RL78_OP_REL (ABS8UW,       0,  8, 0, unsigned, FALSE),
-  RL78_OP_REL (ABS32_REV,    2, 32, 0, dont,     FALSE),
-  RL78_OP_REL (ABS16_REV,    1, 16, 0, dont,     FALSE),
+  RL78_OP_REL (ABS32,	     2, 32, 0, dont,	 FALSE),
+  RL78_OP_REL (ABS24S,	     2, 24, 0, signed,	 FALSE),
+  RL78_OP_REL (ABS16,	     1, 16, 0, dont,	 FALSE),
+  RL78_OP_REL (ABS16U,	     1, 16, 0, unsigned, FALSE),
+  RL78_OP_REL (ABS16S,	     1, 16, 0, signed,	 FALSE),
+  RL78_OP_REL (ABS8,	     0,	 8, 0, dont,	 FALSE),
+  RL78_OP_REL (ABS8U,	     0,	 8, 0, unsigned, FALSE),
+  RL78_OP_REL (ABS8S,	     0,	 8, 0, signed,	 FALSE),
+  RL78_OP_REL (ABS24S_PCREL, 2, 24, 0, signed,	 TRUE),
+  RL78_OP_REL (ABS16S_PCREL, 1, 16, 0, signed,	 TRUE),
+  RL78_OP_REL (ABS8S_PCREL,  0,	 8, 0, signed,	 TRUE),
+  RL78_OP_REL (ABS16UL,	     1, 16, 0, unsigned, FALSE),
+  RL78_OP_REL (ABS16UW,	     1, 16, 0, unsigned, FALSE),
+  RL78_OP_REL (ABS8UL,	     0,	 8, 0, unsigned, FALSE),
+  RL78_OP_REL (ABS8UW,	     0,	 8, 0, unsigned, FALSE),
+  RL78_OP_REL (ABS32_REV,    2, 32, 0, dont,	 FALSE),
+  RL78_OP_REL (ABS16_REV,    1, 16, 0, dont,	 FALSE),
 
 #define STACK_REL_P(x) ((x) <= R_RL78_ABS16_REV && (x) >= R_RL78_ABS32)
 
@@ -182,14 +183,14 @@ static reloc_howto_type rl78_elf_howto_table [] =
   EMPTY_HOWTO (0x7e),
   EMPTY_HOWTO (0x7f),
 
-  RL78_OP_REL (SYM,       2, 32, 0, dont, FALSE),
-  RL78_OP_REL (OPneg,     2, 32, 0, dont, FALSE),
-  RL78_OP_REL (OPadd,     2, 32, 0, dont, FALSE),
-  RL78_OP_REL (OPsub,     2, 32, 0, dont, FALSE),
-  RL78_OP_REL (OPmul,     2, 32, 0, dont, FALSE),
-  RL78_OP_REL (OPdiv,     2, 32, 0, dont, FALSE),
-  RL78_OP_REL (OPshla,    2, 32, 0, dont, FALSE),
-  RL78_OP_REL (OPshra,    2, 32, 0, dont, FALSE),
+  RL78_OP_REL (SYM,	  2, 32, 0, dont, FALSE),
+  RL78_OP_REL (OPneg,	  2, 32, 0, dont, FALSE),
+  RL78_OP_REL (OPadd,	  2, 32, 0, dont, FALSE),
+  RL78_OP_REL (OPsub,	  2, 32, 0, dont, FALSE),
+  RL78_OP_REL (OPmul,	  2, 32, 0, dont, FALSE),
+  RL78_OP_REL (OPdiv,	  2, 32, 0, dont, FALSE),
+  RL78_OP_REL (OPshla,	  2, 32, 0, dont, FALSE),
+  RL78_OP_REL (OPshra,	  2, 32, 0, dont, FALSE),
   RL78_OP_REL (OPsctsize, 2, 32, 0, dont, FALSE),
   EMPTY_HOWTO (0x89),
   EMPTY_HOWTO (0x8a),
@@ -198,11 +199,11 @@ static reloc_howto_type rl78_elf_howto_table [] =
   RL78_OP_REL (OPscttop,  2, 32, 0, dont, FALSE),
   EMPTY_HOWTO (0x8e),
   EMPTY_HOWTO (0x8f),
-  RL78_OP_REL (OPand,     2, 32, 0, dont, FALSE),
-  RL78_OP_REL (OPor,      2, 32, 0, dont, FALSE),
-  RL78_OP_REL (OPxor,     2, 32, 0, dont, FALSE),
-  RL78_OP_REL (OPnot,     2, 32, 0, dont, FALSE),
-  RL78_OP_REL (OPmod,     2, 32, 0, dont, FALSE),
+  RL78_OP_REL (OPand,	  2, 32, 0, dont, FALSE),
+  RL78_OP_REL (OPor,	  2, 32, 0, dont, FALSE),
+  RL78_OP_REL (OPxor,	  2, 32, 0, dont, FALSE),
+  RL78_OP_REL (OPnot,	  2, 32, 0, dont, FALSE),
+  RL78_OP_REL (OPmod,	  2, 32, 0, dont, FALSE),
   RL78_OP_REL (OPromtop,  2, 32, 0, dont, FALSE),
   RL78_OP_REL (OPramtop,  2, 32, 0, dont, FALSE)
 };
@@ -212,7 +213,7 @@ static reloc_howto_type rl78_elf_howto_table [] =
 struct rl78_reloc_map
 {
   bfd_reloc_code_real_type  bfd_reloc_val;
-  unsigned int              rl78_reloc_val;
+  unsigned int		    rl78_reloc_val;
 };
 
 static const struct rl78_reloc_map rl78_reloc_map [] =
@@ -242,6 +243,7 @@ static const struct rl78_reloc_map rl78_reloc_map [] =
   { BFD_RELOC_RL78_ABS16UL,	R_RL78_ABS16UL },
   { BFD_RELOC_RL78_ABS16UW,	R_RL78_ABS16UW },
   { BFD_RELOC_RL78_ABS16U,	R_RL78_ABS16U },
+  { BFD_RELOC_RL78_SADDR,	R_RL78_RH_SADDR },
   { BFD_RELOC_RL78_RELAX,	R_RL78_RH_RELAX }
 };
 
@@ -277,8 +279,8 @@ rl78_reloc_name_lookup (bfd * abfd ATTRIBUTE_UNUSED, const char * r_name)
 /* Set the howto pointer for an RL78 ELF reloc.  */
 
 static void
-rl78_info_to_howto_rela (bfd *               abfd ATTRIBUTE_UNUSED,
-			 arelent *           cache_ptr,
+rl78_info_to_howto_rela (bfd *		     abfd ATTRIBUTE_UNUSED,
+			 arelent *	     cache_ptr,
 			 Elf_Internal_Rela * dst)
 {
   unsigned int r_type;
@@ -286,6 +288,7 @@ rl78_info_to_howto_rela (bfd *               abfd ATTRIBUTE_UNUSED,
   r_type = ELF32_R_TYPE (dst->r_info);
   if (r_type >= (unsigned int) R_RL78_max)
     {
+      /* xgettext:c-format */
       _bfd_error_handler (_("%B: invalid RL78 reloc number: %d"), abfd, r_type);
       r_type = 0;
     }
@@ -293,11 +296,10 @@ rl78_info_to_howto_rela (bfd *               abfd ATTRIBUTE_UNUSED,
 }
 
 static bfd_vma
-get_symbol_value (const char *            name,
-		  bfd_reloc_status_type * status,
+get_symbol_value (const char *		  name,
 		  struct bfd_link_info *  info,
-		  bfd *                   input_bfd,
-		  asection *              input_section,
+		  bfd *			  input_bfd,
+		  asection *		  input_section,
 		  int			  offset)
 {
   struct bfd_link_hash_entry * h;
@@ -311,12 +313,8 @@ get_symbol_value (const char *            name,
       || (h->type != bfd_link_hash_defined
 	  && h->type != bfd_link_hash_defweak))
     {
-      bfd_reloc_status_type res;
-
-      res = info->callbacks->undefined_symbol
+      (*info->callbacks->undefined_symbol)
 	(info, name, input_bfd, input_section, offset, TRUE);
-      if (status)
-	* status = res;
       return 0;
     }
 
@@ -326,44 +324,36 @@ get_symbol_value (const char *            name,
 }
 
 static bfd_vma
-get_romstart (bfd_reloc_status_type * status,
-	      struct bfd_link_info *  info,
-	      bfd *                   abfd,
-	      asection *              sec,
+get_romstart (struct bfd_link_info *  info,
+	      bfd *		      abfd,
+	      asection *	      sec,
 	      int		      offset)
 {
   static bfd_boolean cached = FALSE;
   static bfd_vma     cached_value = 0;
-  static bfd_reloc_status_type cached_status;
 
   if (!cached)
     {
-      cached_value = get_symbol_value ("_start", & cached_status, info, abfd, sec, offset);
+      cached_value = get_symbol_value ("_start", info, abfd, sec, offset);
       cached = TRUE;
     }
-  if (status)
-    * status = cached_status;
   return cached_value;
 }
 
 static bfd_vma
-get_ramstart (bfd_reloc_status_type * status,
-	      struct bfd_link_info *  info,
-	      bfd *                   abfd,
-	      asection *              sec,
+get_ramstart (struct bfd_link_info *  info,
+	      bfd *		      abfd,
+	      asection *	      sec,
 	      int		      offset)
 {
   static bfd_boolean cached = FALSE;
   static bfd_vma     cached_value = 0;
-  static bfd_reloc_status_type cached_status;
 
   if (!cached)
     {
-      cached_value = get_symbol_value ("__datastart", & cached_status, info, abfd, sec, offset);
+      cached_value = get_symbol_value ("__datastart", info, abfd, sec, offset);
       cached = TRUE;
     }
-  if (status)
-    * status = cached_status;
   return cached_value;
 }
 
@@ -375,7 +365,7 @@ static unsigned int rl78_stack_top;
   do						\
     {						\
       if (rl78_stack_top < NUM_STACK_ENTRIES)	\
-        rl78_stack [rl78_stack_top ++] = (val);	\
+	rl78_stack [rl78_stack_top ++] = (val);	\
       else					\
 	_bfd_error_handler (_("Internal Error: RL78 reloc stack overflow")); \
     }						\
@@ -385,12 +375,12 @@ static unsigned int rl78_stack_top;
   do						\
     {						\
       if (rl78_stack_top > 0)			\
-        (dest) = rl78_stack [-- rl78_stack_top];\
+	(dest) = rl78_stack [-- rl78_stack_top];\
       else					\
 	{					\
-          _bfd_error_handler (_("Internal Error: RL78 reloc stack underflow")); \
-          (dest) = 0;				\
-        }					\
+	  _bfd_error_handler (_("Internal Error: RL78 reloc stack underflow")); \
+	  (dest) = 0;				\
+	}					\
     }						\
   while (0)
 
@@ -401,8 +391,8 @@ static unsigned int rl78_stack_top;
 
 static bfd_vma
 rl78_compute_complex_reloc (unsigned long  r_type,
-			    bfd_vma        symval,
-			    asection *     input_section)
+			    bfd_vma	   symval,
+			    asection *	   input_section)
 {
   int32_t tmp1, tmp2;
   bfd_vma relocation;
@@ -447,7 +437,7 @@ rl78_compute_complex_reloc (unsigned long  r_type,
     case R_RL78_SYM:
       RL78_STACK_PUSH (symval);
       return 0;
-      
+
     case R_RL78_OPneg:
       RL78_STACK_POP (tmp1);
       tmp1 = - tmp1;
@@ -556,10 +546,10 @@ rl78_special_reloc (bfd *      input_bfd,
 		    bfd *      output_bfd ATTRIBUTE_UNUSED,
 		    char **    error_message ATTRIBUTE_UNUSED)
 {
-  bfd_reloc_status_type  r = bfd_reloc_ok;
-  bfd_vma                relocation = 0;
-  unsigned long          r_type = reloc->howto->type;
-  bfd_byte *             contents = data;
+  bfd_reloc_status_type	 r = bfd_reloc_ok;
+  bfd_vma		 relocation = 0;
+  unsigned long		 r_type = reloc->howto->type;
+  bfd_byte *		 contents = data;
 
   /* If necessary, compute the symbolic value of the relocation.  */
   switch (r_type)
@@ -572,12 +562,12 @@ rl78_special_reloc (bfd *      input_bfd,
 	break;
 
     case R_RL78_OPromtop:
-      relocation = get_romstart (&r, NULL, input_bfd, input_section,
+      relocation = get_romstart (NULL, input_bfd, input_section,
 				 reloc->address);
       break;
 
     case R_RL78_OPramtop:
-      relocation = get_ramstart (&r, NULL, input_bfd, input_section,
+      relocation = get_ramstart (NULL, input_bfd, input_section,
 				 reloc->address);
       break;
     }
@@ -683,49 +673,45 @@ rl78_special_reloc (bfd *      input_bfd,
 
 static bfd_boolean
 rl78_elf_relocate_section
-    (bfd *                   output_bfd,
+    (bfd *		     output_bfd,
      struct bfd_link_info *  info,
-     bfd *                   input_bfd,
-     asection *              input_section,
-     bfd_byte *              contents,
+     bfd *		     input_bfd,
+     asection *		     input_section,
+     bfd_byte *		     contents,
      Elf_Internal_Rela *     relocs,
-     Elf_Internal_Sym *      local_syms,
-     asection **             local_sections)
+     Elf_Internal_Sym *	     local_syms,
+     asection **	     local_sections)
 {
-  Elf_Internal_Shdr *           symtab_hdr;
+  Elf_Internal_Shdr *		symtab_hdr;
   struct elf_link_hash_entry ** sym_hashes;
-  Elf_Internal_Rela *           rel;
-  Elf_Internal_Rela *           relend;
-  bfd *dynobj;
+  Elf_Internal_Rela *		rel;
+  Elf_Internal_Rela *		relend;
   asection *splt;
 
   symtab_hdr = & elf_tdata (input_bfd)->symtab_hdr;
   sym_hashes = elf_sym_hashes (input_bfd);
   relend     = relocs + input_section->reloc_count;
 
-  dynobj = elf_hash_table (info)->dynobj;
-  splt = NULL;
-  if (dynobj != NULL)
-    splt = bfd_get_linker_section (dynobj, ".plt");
+  splt = elf_hash_table (info)->splt;
 
   for (rel = relocs; rel < relend; rel ++)
     {
-      reloc_howto_type *           howto;
-      unsigned long                r_symndx;
-      Elf_Internal_Sym *           sym;
-      asection *                   sec;
+      reloc_howto_type *	   howto;
+      unsigned long		   r_symndx;
+      Elf_Internal_Sym *	   sym;
+      asection *		   sec;
       struct elf_link_hash_entry * h;
-      bfd_vma                      relocation;
-      bfd_reloc_status_type        r;
-      const char *                 name = NULL;
-      bfd_boolean                  unresolved_reloc = TRUE;
-      int                          r_type;
+      bfd_vma			   relocation;
+      bfd_reloc_status_type	   r;
+      const char *		   name = NULL;
+      bfd_boolean		   unresolved_reloc = TRUE;
+      int			   r_type;
 
       r_type = ELF32_R_TYPE (rel->r_info);
       r_symndx = ELF32_R_SYM (rel->r_info);
 
       howto  = rl78_elf_howto_table + ELF32_R_TYPE (rel->r_info);
-      h      = NULL;
+      h	     = NULL;
       sym    = NULL;
       sec    = NULL;
       relocation = 0;
@@ -757,12 +743,12 @@ rl78_elf_relocate_section
 	RELOC_AGAINST_DISCARDED_SECTION (info, input_bfd, input_section,
 					 rel, 1, relend, howto, 0, contents);
 
-      if (info->relocatable)
+      if (bfd_link_relocatable (info))
 	{
 	  /* This is a relocatable link.  We don't have to change
-             anything, unless the reloc is against a section symbol,
-             in which case we have to adjust according to where the
-             section symbol winds up in the output section.  */
+	     anything, unless the reloc is against a section symbol,
+	     in which case we have to adjust according to where the
+	     section symbol winds up in the output section.  */
 	  if (sym != NULL && ELF_ST_TYPE (sym->st_info) == STT_SECTION)
 	    rel->r_addend += sec->output_offset;
 	  continue;
@@ -1045,7 +1031,7 @@ rl78_elf_relocate_section
 	      break;
 	    }
 	  break;
-	  
+
 	case R_RL78_SYM:
 	  if (r_symndx < symtab_hdr->sh_info)
 	    relocation = sec->output_section->vma + sec->output_offset
@@ -1067,12 +1053,12 @@ rl78_elf_relocate_section
 	  break;
 
 	case R_RL78_OPromtop:
-	  relocation = get_romstart (&r, info, input_bfd, input_section, rel->r_offset);
+	  relocation = get_romstart (info, input_bfd, input_section, rel->r_offset);
 	  (void) rl78_compute_complex_reloc (r_type, relocation, input_section);
 	  break;
 
 	case R_RL78_OPramtop:
-	  relocation = get_ramstart (&r, info, input_bfd, input_section, rel->r_offset);
+	  relocation = get_ramstart (info, input_bfd, input_section, rel->r_offset);
 	  (void) rl78_compute_complex_reloc (r_type, relocation, input_section);
 	  break;
 
@@ -1091,45 +1077,47 @@ rl78_elf_relocate_section
 	      /* Catch the case of a missing function declaration
 		 and emit a more helpful error message.  */
 	      if (r_type == R_RL78_DIR24S_PCREL)
+		/* xgettext:c-format */
 		msg = _("%B(%A): error: call to undefined function '%s'");
 	      else
-		r = info->callbacks->reloc_overflow
+		(*info->callbacks->reloc_overflow)
 		  (info, (h ? &h->root : NULL), name, howto->name, (bfd_vma) 0,
 		   input_bfd, input_section, rel->r_offset);
 	      break;
 
 	    case bfd_reloc_undefined:
-	      r = info->callbacks->undefined_symbol
-		(info, name, input_bfd, input_section, rel->r_offset,
-		 TRUE);
+	      (*info->callbacks->undefined_symbol)
+		(info, name, input_bfd, input_section, rel->r_offset, TRUE);
 	      break;
 
 	    case bfd_reloc_other:
+	      /* xgettext:c-format */
 	      msg = _("%B(%A): warning: unaligned access to symbol '%s' in the small data area");
 	      break;
 
 	    case bfd_reloc_outofrange:
+	      /* xgettext:c-format */
 	      msg = _("%B(%A): internal error: out of range error");
 	      break;
 
 	    case bfd_reloc_notsupported:
+	      /* xgettext:c-format */
 	      msg = _("%B(%A): internal error: unsupported relocation error");
 	      break;
 
 	    case bfd_reloc_dangerous:
+	      /* xgettext:c-format */
 	      msg = _("%B(%A): internal error: dangerous relocation");
 	      break;
 
 	    default:
+	      /* xgettext:c-format */
 	      msg = _("%B(%A): internal error: unknown error");
 	      break;
 	    }
 
 	  if (msg)
 	    _bfd_error_handler (msg, input_bfd, input_section, name);
-
-	  if (! r)
-	    return FALSE;
 	}
     }
 
@@ -1172,8 +1160,9 @@ rl78_cpu_name (flagword flags)
    object file when linking.  */
 
 static bfd_boolean
-rl78_elf_merge_private_bfd_data (bfd * ibfd, bfd * obfd)
+rl78_elf_merge_private_bfd_data (bfd *ibfd, struct bfd_link_info *info)
 {
+  bfd *obfd = info->output_bfd;
   flagword new_flags;
   flagword old_flags;
   bfd_boolean error = FALSE;
@@ -1208,10 +1197,11 @@ rl78_elf_merge_private_bfd_data (bfd * ibfd, bfd * obfd)
 		     but that is still incompatible with the G10 ABI.  */
 		  error = TRUE;
 
-		  (*_bfd_error_handler)
-		    (_("RL78 ABI conflict: G10 file %s cannot be linked with %s file %s"),
-		     bfd_get_filename (ibfd),
-		     rl78_cpu_name (out_cpu), bfd_get_filename (obfd));
+		  _bfd_error_handler
+		    /* xgettext:c-format */
+		    (_("RL78 ABI conflict: G10 file %B cannot be linked"
+		       " with %s file %B"),
+		     ibfd, rl78_cpu_name (out_cpu), obfd);
 		}
 	      else
 		{
@@ -1224,26 +1214,29 @@ rl78_elf_merge_private_bfd_data (bfd * ibfd, bfd * obfd)
 	    {
 	      error = TRUE;
 
-	      (*_bfd_error_handler)
-		(_("RL78 ABI conflict: cannot link %s file %s with %s file %s"),
-		 rl78_cpu_name (in_cpu),  bfd_get_filename (ibfd),
-		 rl78_cpu_name (out_cpu), bfd_get_filename (obfd));
+	      _bfd_error_handler
+		/* xgettext:c-format */
+		(_("RL78 ABI conflict: cannot link %s file %B with %s file %B"),
+		 rl78_cpu_name (in_cpu),  ibfd,
+		 rl78_cpu_name (out_cpu), obfd);
 	    }
 	}
 
       if (changed_flags & E_FLAG_RL78_64BIT_DOUBLES)
 	{
-	  (*_bfd_error_handler)
+	  _bfd_error_handler
 	    (_("RL78 merge conflict: cannot link 32-bit and 64-bit objects together"));
 
 	  if (old_flags & E_FLAG_RL78_64BIT_DOUBLES)
-	    (*_bfd_error_handler) (_("- %s is 64-bit, %s is not"),
-				   bfd_get_filename (obfd), bfd_get_filename (ibfd));
+	    /* xgettext:c-format */
+	    _bfd_error_handler (_("- %B is 64-bit, %B is not"),
+				obfd, ibfd);
 	  else
-	    (*_bfd_error_handler) (_("- %s is 64-bit, %s is not"),
-				   bfd_get_filename (ibfd), bfd_get_filename (obfd));
+	    /* xgettext:c-format */
+	    _bfd_error_handler (_("- %B is 64-bit, %B is not"),
+				ibfd, obfd);
 	  error = TRUE;
-	}    
+	}
     }
 
   return !error;
@@ -1296,26 +1289,25 @@ rl78_elf_object_p (bfd * abfd)
 
 static bfd_boolean
 rl78_elf_check_relocs
-    (bfd *                     abfd,
+    (bfd *		       abfd,
      struct bfd_link_info *    info,
-     asection *                sec,
+     asection *		       sec,
      const Elf_Internal_Rela * relocs)
 {
-  Elf_Internal_Shdr *           symtab_hdr;
+  Elf_Internal_Shdr *		symtab_hdr;
   struct elf_link_hash_entry ** sym_hashes;
-  const Elf_Internal_Rela *     rel;
-  const Elf_Internal_Rela *     rel_end;
+  const Elf_Internal_Rela *	rel;
+  const Elf_Internal_Rela *	rel_end;
   bfd_vma *local_plt_offsets;
   asection *splt;
   bfd *dynobj;
 
-  if (info->relocatable)
+  if (bfd_link_relocatable (info))
     return TRUE;
 
   symtab_hdr = &elf_tdata (abfd)->symtab_hdr;
   sym_hashes = elf_sym_hashes (abfd);
   local_plt_offsets = elf_local_got_offsets (abfd);
-  splt = NULL;
   dynobj = elf_hash_table(info)->dynobj;
 
   rel_end = relocs + sec->reloc_count;
@@ -1327,41 +1319,35 @@ rl78_elf_check_relocs
 
       r_symndx = ELF32_R_SYM (rel->r_info);
       if (r_symndx < symtab_hdr->sh_info)
-        h = NULL;
+	h = NULL;
       else
 	{
 	  h = sym_hashes[r_symndx - symtab_hdr->sh_info];
 	  while (h->root.type == bfd_link_hash_indirect
 		 || h->root.type == bfd_link_hash_warning)
 	    h = (struct elf_link_hash_entry *) h->root.u.i.link;
-
-	  /* PR15323, ref flags aren't set for references in the same
-	     object.  */
-	  h->root.non_ir_ref = 1;
 	}
 
       switch (ELF32_R_TYPE (rel->r_info))
-        {
+	{
 	  /* This relocation describes a 16-bit pointer to a function.
 	     We may need to allocate a thunk in low memory; reserve memory
 	     for it now.  */
 	case R_RL78_DIR16S:
 	  if (dynobj == NULL)
 	    elf_hash_table (info)->dynobj = dynobj = abfd;
+	  splt = elf_hash_table (info)->splt;
 	  if (splt == NULL)
 	    {
-	      splt = bfd_get_linker_section (dynobj, ".plt");
-	      if (splt == NULL)
-		{
-		  flagword flags = (SEC_ALLOC | SEC_LOAD | SEC_HAS_CONTENTS
-				    | SEC_IN_MEMORY | SEC_LINKER_CREATED
-				    | SEC_READONLY | SEC_CODE);
-		  splt = bfd_make_section_anyway_with_flags (dynobj, ".plt",
-							     flags);
-		  if (splt == NULL
-		      || ! bfd_set_section_alignment (dynobj, splt, 1))
-		    return FALSE;
-		}
+	      flagword flags = (SEC_ALLOC | SEC_LOAD | SEC_HAS_CONTENTS
+				| SEC_IN_MEMORY | SEC_LINKER_CREATED
+				| SEC_READONLY | SEC_CODE);
+	      splt = bfd_make_section_anyway_with_flags (dynobj, ".plt",
+							 flags);
+	      elf_hash_table (info)->splt = splt;
+	      if (splt == NULL
+		  || ! bfd_set_section_alignment (dynobj, splt, 1))
+		return FALSE;
 	    }
 
 	  if (h != NULL)
@@ -1391,7 +1377,7 @@ rl78_elf_check_relocs
 	      splt->size += 4;
 	    }
 	  break;
-        }
+	}
     }
 
   return TRUE;
@@ -1401,7 +1387,7 @@ rl78_elf_check_relocs
 
 static bfd_boolean
 rl78_elf_finish_dynamic_sections (bfd *abfd ATTRIBUTE_UNUSED,
-                                  struct bfd_link_info *info)
+				  struct bfd_link_info *info)
 {
   bfd *dynobj;
   asection *splt;
@@ -1418,8 +1404,9 @@ rl78_elf_finish_dynamic_sections (bfd *abfd ATTRIBUTE_UNUSED,
   if (info->relax_trip > 0)
     return TRUE;
 
-  if ((dynobj = elf_hash_table (info)->dynobj) != NULL
-      && (splt = bfd_get_linker_section (dynobj, ".plt")) != NULL)
+  dynobj = elf_hash_table (info)->dynobj;
+  splt = elf_hash_table (info)->splt;
+  if (dynobj != NULL && splt != NULL)
     {
       bfd_byte *contents = splt->contents;
       unsigned int i, size = splt->size;
@@ -1436,19 +1423,19 @@ rl78_elf_finish_dynamic_sections (bfd *abfd ATTRIBUTE_UNUSED,
 
 static bfd_boolean
 rl78_elf_always_size_sections (bfd *output_bfd ATTRIBUTE_UNUSED,
-                               struct bfd_link_info *info)
+			       struct bfd_link_info *info)
 {
   bfd *dynobj;
   asection *splt;
 
-  if (info->relocatable)
+  if (bfd_link_relocatable (info))
     return TRUE;
 
   dynobj = elf_hash_table (info)->dynobj;
   if (dynobj == NULL)
     return TRUE;
 
-  splt = bfd_get_linker_section (dynobj, ".plt");
+  splt = elf_hash_table (info)->splt;
   BFD_ASSERT (splt != NULL);
 
   splt->contents = (bfd_byte *) bfd_zalloc (dynobj, splt->size);
@@ -1518,9 +1505,9 @@ rl78_relax_plt_realloc (struct elf_link_hash_entry *h, void * xdata)
 
 static bfd_boolean
 rl78_elf_relax_plt_section (bfd *dynobj,
-                            asection *splt,
-                            struct bfd_link_info *info,
-                            bfd_boolean *again)
+			    asection *splt,
+			    struct bfd_link_info *info,
+			    bfd_boolean *again)
 {
   struct relax_plt_data relax_plt_data;
   bfd *ibfd;
@@ -1528,7 +1515,7 @@ rl78_elf_relax_plt_section (bfd *dynobj,
   /* Assume nothing changes.  */
   *again = FALSE;
 
-  if (info->relocatable)
+  if (bfd_link_relocatable (info))
     return TRUE;
 
   /* We only relax the .plt section at the moment.  */
@@ -1635,7 +1622,7 @@ rl78_elf_relax_plt_section (bfd *dynobj,
 	  for (idx = 0; idx < nlocals; ++idx)
 	    if (local_plt_offsets[idx] != (bfd_vma) -1)
 	      {
-	        local_plt_offsets[idx] = entry;
+		local_plt_offsets[idx] = entry;
 		entry += 4;
 	      }
 	}
@@ -1651,14 +1638,14 @@ elf32_rl78_relax_delete_bytes (bfd *abfd, asection *sec, bfd_vma addr, int count
 			       Elf_Internal_Rela *alignment_rel, int force_snip)
 {
   Elf_Internal_Shdr * symtab_hdr;
-  unsigned int        sec_shndx;
-  bfd_byte *          contents;
+  unsigned int	      sec_shndx;
+  bfd_byte *	      contents;
   Elf_Internal_Rela * irel;
   Elf_Internal_Rela * irelend;
   Elf_Internal_Sym *  isym;
   Elf_Internal_Sym *  isymend;
-  bfd_vma             toaddr;
-  unsigned int        symcount;
+  bfd_vma	      toaddr;
+  unsigned int	      symcount;
   struct elf_link_hash_entry ** sym_hashes;
   struct elf_link_hash_entry ** end_hashes;
 
@@ -1829,16 +1816,16 @@ reloc_bubblesort (Elf_Internal_Rela * r, int count)
 			 lrel, abfd, sec, link_info, scale)
 
 static bfd_vma
-rl78_offset_for_reloc (bfd *                    abfd,
-		       Elf_Internal_Rela *      rel,
-		       Elf_Internal_Shdr *      symtab_hdr,
+rl78_offset_for_reloc (bfd *			abfd,
+		       Elf_Internal_Rela *	rel,
+		       Elf_Internal_Shdr *	symtab_hdr,
 		       Elf_External_Sym_Shndx * shndx_buf ATTRIBUTE_UNUSED,
-		       Elf_Internal_Sym *       intsyms,
-		       Elf_Internal_Rela **     lrel,
-		       bfd *                    input_bfd,
-		       asection *               input_section,
-		       struct bfd_link_info *   info,
-		       int *                    scale)
+		       Elf_Internal_Sym *	intsyms,
+		       Elf_Internal_Rela **	lrel,
+		       bfd *			input_bfd,
+		       asection *		input_section,
+		       struct bfd_link_info *	info,
+		       int *			scale)
 {
   bfd_vma symval;
 
@@ -1931,12 +1918,12 @@ rl78_offset_for_reloc (bfd *                    abfd,
 	  break;
 
 	case R_RL78_OPromtop:
-	  symval = get_romstart (NULL, info, input_bfd, input_section, rel->r_offset);
+	  symval = get_romstart (info, input_bfd, input_section, rel->r_offset);
 	  (void) rl78_compute_complex_reloc (r_type, symval, input_section);
 	  break;
 
 	case R_RL78_OPramtop:
-	  symval = get_ramstart (NULL, info, input_bfd, input_section, rel->r_offset);
+	  symval = get_ramstart (info, input_bfd, input_section, rel->r_offset);
 	  (void) rl78_compute_complex_reloc (r_type, symval, input_section);
 	  break;
 
@@ -1970,10 +1957,19 @@ rl78_offset_for_reloc (bfd *                    abfd,
 	case R_RL78_ABS8UW:
 	  *scale = 2;
 	  goto reloc_computes_value;
-	  
+
 	default:
 	reloc_computes_value:
-	  symval = rl78_compute_complex_reloc (r_type, 0, input_section);
+	  symval = rl78_compute_complex_reloc (r_type, symval, input_section);
+	  /* Fall through.  */
+	case R_RL78_DIR32:
+	case R_RL78_DIR24S:
+	case R_RL78_DIR16:
+	case R_RL78_DIR16U:
+	case R_RL78_DIR16S:
+	case R_RL78_DIR24S_PCREL:
+	case R_RL78_DIR16S_PCREL:
+	case R_RL78_DIR8S_PCREL:
 	  if (lrel)
 	    *lrel = rel;
 	  return symval;
@@ -2052,10 +2048,10 @@ struct {
 
 static bfd_boolean
 rl78_elf_relax_section
-    (bfd *                  abfd,
-     asection *             sec,
+    (bfd *		    abfd,
+     asection *		    sec,
      struct bfd_link_info * link_info,
-     bfd_boolean *          again)
+     bfd_boolean *	    again)
 {
   Elf_Internal_Shdr * symtab_hdr;
   Elf_Internal_Shdr * shndx_hdr;
@@ -2065,8 +2061,8 @@ rl78_elf_relax_section
   Elf_Internal_Rela * srel;
   Elf_Internal_Rela * irelend;
   Elf_Internal_Rela * next_alignment;
-  bfd_byte *          contents = NULL;
-  bfd_byte *          free_contents = NULL;
+  bfd_byte *	      contents = NULL;
+  bfd_byte *	      free_contents = NULL;
   Elf_Internal_Sym *  intsyms = NULL;
   Elf_Internal_Sym *  free_intsyms = NULL;
   Elf_External_Sym_Shndx * shndx_buf = NULL;
@@ -2087,14 +2083,17 @@ rl78_elf_relax_section
   /* We don't have to do anything for a relocatable link, if
      this section does not have relocs, or if this is not a
      code section.  */
-  if (link_info->relocatable
+  if (bfd_link_relocatable (link_info)
       || (sec->flags & SEC_RELOC) == 0
       || sec->reloc_count == 0
       || (sec->flags & SEC_CODE) == 0)
     return TRUE;
 
-  symtab_hdr = &elf_tdata (abfd)->symtab_hdr;
-  shndx_hdr = &elf_tdata (abfd)->symtab_shndx_hdr;
+  symtab_hdr = & elf_symtab_hdr (abfd);
+  if (elf_symtab_shndx_list (abfd))
+    shndx_hdr = & elf_symtab_shndx_list (abfd)->hdr;
+  else
+    shndx_hdr = NULL;
 
   /* Get the section contents.  */
   if (elf_section_data (sec)->this_hdr.contents != NULL)
@@ -2117,7 +2116,7 @@ rl78_elf_relax_section
       symtab_hdr->contents = (bfd_byte *) intsyms;
     }
 
-  if (shndx_hdr->sh_size != 0)
+  if (shndx_hdr && shndx_hdr->sh_size != 0)
     {
       bfd_size_type amt;
 
@@ -2245,9 +2244,9 @@ rl78_elf_relax_section
 	 reloc entry (such as gp-rel symbols).  */
 
       /* Get the value of the symbol referred to by the reloc.  Just
-         in case this is the last reloc in the list, use the RL's
-         addend to choose between this reloc (no addend) or the next
-         (yes addend, which means at least one following reloc).  */
+	 in case this is the last reloc in the list, use the RL's
+	 addend to choose between this reloc (no addend) or the next
+	 (yes addend, which means at least one following reloc).  */
 
       /* srel points to the "current" reloction for this insn -
 	 actually the last reloc for a given operand, which is the one
@@ -2269,7 +2268,7 @@ rl78_elf_relax_section
 	elf32_rl78_relax_delete_bytes (abfd, sec, (insn - contents) + offset, nbytes, next_alignment, 0);
 
 #define SNIP(offset, nbytes, newtype)					\
-        SNIPNR (offset, nbytes);					\
+	SNIPNR (offset, nbytes);					\
 	srel->r_info = ELF32_R_INFO (ELF32_R_SYM (srel->r_info), newtype)
 
       /* The order of these bit tests must match the order that the
@@ -2312,6 +2311,27 @@ rl78_elf_relax_section
 
 	  switch (insn[0])
 	    {
+	    case 0xdc: /* BC */
+	    case 0xdd: /* BZ */
+	    case 0xde: /* BNC */
+	    case 0xdf: /* BNZ */
+	      if (insn[1] == 0x03 && insn[2] == 0xee /* BR */
+		  && (srel->r_offset - irel->r_offset) > 1) /* a B<c> without its own reloc */
+		{
+		  /* This is a "long" conditional as generated by gas:
+		     DC 03 EE ad.dr  */
+		  if (pcrel < 127
+		      && pcrel > -127)
+		    {
+		      insn[0] ^= 0x02; /* invert conditional */
+		      SNIPNR (4, 1);
+		      SNIP (1, 2, R_RL78_DIR8S_PCREL);
+		      insn[1] = pcrel;
+		      *again = TRUE;
+		    }
+		}
+	      break;
+
 	    case 0xec: /* BR !!abs20 */
 
 	      if (pcrel < 127
@@ -2327,7 +2347,7 @@ rl78_elf_relax_section
 		  insn[0] = 0xed;
 		  insn[1] = symval & 0xff;
 		  insn[2] = symval >> 8;
-		  SNIP (2, 1, R_RL78_DIR16S);
+		  SNIP (2, 1, R_RL78_DIR16U);
 		  *again = TRUE;
 		}
 	      else if (pcrel < 32767
@@ -2359,7 +2379,7 @@ rl78_elf_relax_section
 		  insn[0] = 0xfd;
 		  insn[1] = symval & 0xff;
 		  insn[2] = symval >> 8;
-		  SNIP (2, 1, R_RL78_DIR16S);
+		  SNIP (2, 1, R_RL78_DIR16U);
 		  *again = TRUE;
 		}
 	      else if (pcrel < 32767
@@ -2382,6 +2402,25 @@ rl78_elf_relax_section
 		 here anyway. */
 	      switch (insn[1])
 		{
+		case 0xd3: /* BNH */
+		case 0xc3: /* BH */
+		  if (insn[2] == 0x03 && insn[3] == 0xee
+		      && (srel->r_offset - irel->r_offset) > 2) /* a B<c> without its own reloc */
+		    {
+		      /* Another long branch by gas:
+			 61 D3 03 EE ad.dr  */
+		      if (pcrel < 127
+			  && pcrel > -127)
+			{
+			  insn[1] ^= 0x10; /* invert conditional */
+			  SNIPNR (5, 1);
+			  SNIP (2, 2, R_RL78_DIR8S_PCREL);
+			  insn[2] = pcrel;
+			  *again = TRUE;
+			}
+		    }
+		  break;
+
 		case 0xc8: /* SKC */
 		  if (insn[2] == 0xef)
 		    {
@@ -2435,7 +2474,7 @@ rl78_elf_relax_section
 	}
 
       if ((irel->r_addend &  RL78_RELAXA_MASK) == RL78_RELAXA_ADDR16
-          && nrelocs > 0)
+	  && nrelocs > 0)
 	{
 	  /*----------------------------------------------------------------------*/
 	  /* Some insns have both a 16-bit address operand and an 8-bit
@@ -2443,7 +2482,7 @@ rl78_elf_relax_section
 
 	     Address		16-bit operand	SADDR range	SFR range
 	     FFF00-FFFFF	0xff00-0xffff	0x00-0xff
-	     FFE20-FFF1F	0xfe20-0xff1f	 		0x00-0xff
+	     FFE20-FFF1F	0xfe20-0xff1f			0x00-0xff
 
 	     The RELAX_ADDR16[] array has the insn encodings for the
 	     16-bit operand version, as well as the SFR and SADDR
@@ -2538,7 +2577,7 @@ rl78_elf_relax_section
 #define elf_info_to_howto			rl78_info_to_howto_rela
 #define elf_backend_object_p			rl78_elf_object_p
 #define elf_backend_relocate_section		rl78_elf_relocate_section
-#define elf_symbol_leading_char                 ('_')
+#define elf_symbol_leading_char			('_')
 #define elf_backend_can_gc_sections		1
 
 #define bfd_elf32_bfd_reloc_type_lookup		rl78_reloc_type_lookup
@@ -2548,7 +2587,7 @@ rl78_elf_relax_section
 #define bfd_elf32_bfd_print_private_bfd_data	rl78_elf_print_private_bfd_data
 
 #define bfd_elf32_bfd_relax_section		rl78_elf_relax_section
-#define elf_backend_check_relocs                rl78_elf_check_relocs
+#define elf_backend_check_relocs		rl78_elf_check_relocs
 #define elf_backend_always_size_sections \
   rl78_elf_always_size_sections
 #define elf_backend_finish_dynamic_sections \
